#include "searchedit.h"

SearchEdit::SearchEdit(QWidget *parent) : KSearchLineEdit(parent)
{
    this->setFixedSize(200, 40);
    this->setClearButtonEnabled(true);
    this->setFocusPolicy(Qt::ClickFocus);
    this->setContextMenuPolicy(Qt::DefaultContextMenu);

    connect(this, &KSearchLineEdit::textChanged, this, &SearchEdit::onTextChanged);
    connect(this, &KSearchLineEdit::returnPressed, this, &SearchEdit::onReturnPressed);
}

SearchEdit::~SearchEdit()
{
    if (m_searchServer != nullptr) {
        delete m_searchServer;
        m_searchServer = nullptr;
    }
}

void SearchEdit::focusInEvent(QFocusEvent *event)
{
    Q_EMIT sigEditFocusIn();
    return QLineEdit::focusInEvent(event);
}

void SearchEdit::focusOutEvent(QFocusEvent *event)
{
    Q_EMIT sigEditFocusOut();
    return QLineEdit::focusOutEvent(event);
}

void SearchEdit::onTextChanged()
{
    if (!m_mutex.try_lock()) {
        return;
    }
    if (m_searchServer == nullptr) {
        qWarning() << "search server is not ready!";
        return;
    }
    m_query = this->text().trimmed();
    m_mutex.unlock();
    if (m_query.isEmpty()) {
        return;
    }

    auto search = [this]() {
        if (!m_mutex.try_lock()) {
            return;
        }
        m_resultMap.clear();
        m_searchServer->search(m_resultMap, m_query);
        Q_EMIT sigSearchKeyword(m_query);
        Q_EMIT sigSearchResult(m_resultMap);
        m_mutex.unlock();
    };
    std::thread searchThread(search);
    searchThread.detach();
}

void SearchEdit::onReturnPressed()
{
    onTextChanged();
}

void SearchEdit::onSearchInstalled(bool isUpdated, QString searchInfoPath)
{
    qInfo() << "searchInfo installed";

    if (!m_mutex.try_lock()) {
        return;
    }
    if (m_searchServer != nullptr) {
        this->clear();
        delete m_searchServer;
        m_searchServer = nullptr;
    }
    m_searchServer = new SearchServer();
    m_searchServer->setSearchDbPath(searchInfoPath);
    if (m_searchServer) {
        if (m_searchServer->init() < 0) {
            qWarning() << "load search server fail";
            Q_EMIT sigSearchInitFail();
            return;
        }
    }
    m_mutex.unlock();

    if (!isUpdated) {
        qInfo() << "use old searchInfo";
        Q_EMIT sigSearchLineShow();
    } else {
        if (m_searchServer) {
            qInfo() << "use new searchInfo";
            auto searchInit = [this]() {
                m_mutex.lock();
                if (m_searchServer->createIndex() >= 0) {
                    Q_EMIT sigSearchLineShow();
                } else {
                    qWarning() << "search server init fail";
                    Q_EMIT sigSearchInitFail();
                }
                m_mutex.unlock();
            };
            std::thread searchInitThread(searchInit);
            searchInitThread.detach();
        }
    }
}
